<?php

namespace Drupal\site_analysis\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\site_analysis\ContentAnalysis;
use Drupal\site_analysis\FieldAnalysis;
use Drupal\site_analysis\ParagraphAnalysis;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SiteAnalysisForm extends FormBase {

  protected ContentAnalysis $contentAnalysis;

  protected FieldAnalysis $fieldAnalysis;

  protected ParagraphAnalysis $paragraphAnalysis;

  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('site_analysis.content_analysis'),
      $container->get('site_analysis.field_analysis'),
      $container->get('site_analysis.paragraph_analysis'),
    );
  }

  public function __construct(ContentAnalysis $contentAnalysis, FieldAnalysis $fieldAnalysis, ParagraphAnalysis $paragraphAnalysis) {
    $this->contentAnalysis = $contentAnalysis;
    $this->fieldAnalysis = $fieldAnalysis;
    $this->paragraphAnalysis = $paragraphAnalysis;
  }

  public function getFormId(): string {
    return 'drupal_site_analysis_summary_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['#attached']['library'][] = 'site_analysis/site_analysis.admin';

    if (!$this->isAnalyserReady()) {
      return $this->buildNotReadyMessage();
    }

    $contentReport = $this->contentAnalysis->generateContentReport();
    $form['content_report'] = $this->buildContentReport($contentReport);

    $fieldReport = $this->fieldAnalysis->generateFieldReport();
    $form['field_report'] = $this->buildFieldReport($fieldReport);

    $paragraphReport = $this->paragraphAnalysis->generateParagraphReport();
    $form['paragraph_report'] = $this->buildParagraphReport($paragraphReport);

    return $form;
  }

  private function isAnalyserReady(): bool {
    return TRUE; // Placeholder for actual readiness check.
  }

  private function buildNotReadyMessage(): array {
    return [
      '#type' => 'item',
      '#title' => $this->t('Site Analysis is not ready yet.'),
      '#description' => $this->t('Please wait until the analysis is complete.'),
    ];
  }

  private function buildContentReport(array $contentReport): array {
    $report = [
      '#type' => 'details',
      '#title' => $this->t('Content Report'),
      '#open' => TRUE,
    ];

    if (!empty($contentReport['content_types'])) {
      $report['content_types'] = $this->buildTable(
        [$this->t('Content Type ID'), $this->t('Content Type Label')],
        array_map(fn($type) => [
          $type['id'],
          $type['label'],
        ], $contentReport['content_types'])
      );
    }

    if (!empty($contentReport['node_count'])) {
      $report['node_count'] = $this->buildTable(
        [$this->t('Content Type'), $this->t('Node Count')],
        array_map(fn($type, $count) => [
          $type,
          $count,
        ], array_keys($contentReport['node_count']), $contentReport['node_count'])
      );
    }

    if (!empty($contentReport['duplicate_titles'])) {
      $report['duplicate_titles'] = $this->buildTable(
        [$this->t('Content Title'), $this->t('Count')],
        array_map(fn($title) => [
          $title['title'],
          $title['count'],
        ], $contentReport['duplicate_titles'])
      );
    }

    return $report;
  }

  private function buildFieldReport(array $fieldReport) {
    $report = [
      '#type' => 'details',
      '#title' => $this->t('Field Report'),
      '#open' => TRUE,
    ];

    if (!empty($fieldReport['unused_fields'])) {
      $report['content_types'] = $this->buildTable(
        [$this->t('Unused Fields'), $this->t('Unused Fields Label')],
        array_map(fn($type) => [
          $type['id'],
          $type['label'],
        ], $fieldReport['unused_fields'])
      );
    }

    return $report;
  }

  private function buildParagraphReport(array $paragraphReport) {
    $report = [
      '#type' => 'details',
      '#title' => $this->t('Paragraph Report'),
      '#open' => TRUE,
    ];

    if (!empty($paragraphReport['unused_fields'])) {
      $report['content_types'] = $this->buildTable(
        [$this->t('Unused Paragraph'), $this->t('Count')],
        array_map(fn($type) => [
          $type['id'],
          $type['count'],
        ], $paragraphReport['unused_fields'])
      );
    }

    return $report;
  }

  private function buildTable(array $header, array $rows): array {
    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
    ];
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    // TODO: Implement submitForm() method.
  }

}
